
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#define NR_END 1
#define FREE_ARG char *

void nrerror(error_text) 
   char *error_text;
{
  fprintf(stderr,"Runtime error ...\n");
  fprintf(stderr,"%s\n", error_text);
  fprintf(stderr,"...now exiting to system...\n");
  exit(1);
}

/* allocate a float vector with subscript range v[nl..nh] */
float *vector(nl, nh)
     long nl, nh; 
{
  float *v;
  
  v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
  if (!v) nrerror("Allocation failure in vector()");
  return v-nl+NR_END;
}


/* allocate a int vector with subscript range v[nl..nh] */
int *ivector(nl, nh) 
     long nl, nh;
{
  int *v;
  
  v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
  if (!v) nrerror("Allocation failure in ivector()");
  return v-nl+NR_END;
}


/* allocate a unsigned char vector with subscript range v[nl..nh] */
unsigned char *cvector(nl, nh)
     long nl, nh;
{
  unsigned char *v;
  
  v=(unsigned char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned char)));
  if (!v) nrerror("Allocation failure in cvector()");
  return v-nl+NR_END;
}


/* allocate a unsigned long vector with subscript range v[nl..nh] */
unsigned long *lvector(nl, nh)
     long nl, nh;
{
  unsigned long *v;
  
  v=(unsigned long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(long)));
  if (!v) nrerror("Allocation failure in lvector()");
  return v-nl+NR_END;
}


/* allocate a double vector with subscript range v[nl..nh] */
double *dvector(nl, nh)
     long nl, nh;
{
  double *v;
  
  v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
  if (!v) nrerror("Allocation failure in dvector()");
  return v-nl+NR_END;
}


/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
float **matrix(nrl, nrh, ncl, nch)
     long nrl, nrh, ncl, nch;
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  float **m;

  /* allocate pointers to rows */
  m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure 1 in matrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(float *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
  if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i=nrl+1; i <= nrh; i++) m[i] = m[i-1] + ncol;

  /* return pointer to array of pointers to rows */
  return m;
}


/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
double **dmatrix(nrl, nrh, ncl, nch)
     long nrl, nrh,  ncl,  nch;
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  double **m;

  /* allocate pointers to rows */
  m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
  if (!m) nrerror("allocation failure 1 in dmatrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(double *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
  if (!m[nrl]) nrerror("allocation failure 2 in dmatrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i=nrl+1; i <= nrh; i++) m[i] = m[i-1] + ncol;

  /* return pointer to array of pointers to rows */
  return m;
}


/* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
int **imatrix(nrl, nrh, ncl, nch)
     long nrl,  nrh,  ncl,  nch;
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  int **m;

  /* allocate pointers to rows */
  m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
  if (!m) nrerror("allocation failure 1 in imatrix()");
  m += NR_END;
  m -= nrl;

  /* allocate rows and set pointers to them */
  m[nrl]=(int *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
  if (!m[nrl]) nrerror("allocation failure 2 in imatrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i=nrl+1; i <= nrh; i++) m[i] = m[i-1] + ncol;

  /* return pointer to array of pointers to rows */
  return m;
}


/* point to submatrix [newrl..][newcl..] to a[oldrl..oldrh][oldcl..oldch] */
float **submatrix(a, oldrl, oldrh, oldcl, oldch, newrl, newcl)
     float **a;
     long oldrl,  oldrh,  oldcl,  oldch,  newrl,  newcl;
{
  long i, j, nrow = oldrh-oldrl+1, ncol = oldcl-newcl;
  float **m;

  /* allocate array of pointers to rows */
  m = (float **) malloc((size_t) ((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure in submatrix()");
  m += NR_END;
  m -= newrl;

  /* set pointers to rows */
  for (i=oldrl, j=newrl; i <=oldrh; i++,j++) m[j] = a[i]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}

/* allocate a float matrix m[nrl..nrh][ncl..nch] that points to the
 * matrix declared in the standard C manner as a[nrow][ncol], where
 * nrow = nrh-nrl+1 and ncol = nch-ncl+1. The routine should be called
 * with the address &a[0][0] as the first argument */
float **convert_matrix(a, nrl, nrh, ncl, nch)
     float *a;
     long nrl,  nrh,  ncl,  nch;
{
  long i, j, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  float **m;

  /* allocate pointers to rows */
  m = (float **) malloc((size_t) ((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure in convert_matrix()");
  m += NR_END;
  m -= nrl;
 
  /* set pointers to rows */
  m[nrl]=a-ncl;
  for (i=1,j=nrl+1; i<nrow; i++,j++) m[j]=m[j-1]+ncol;

  /* return pointer to array of pointers to rows */
  return m;
}


/* f3tensor not copied !!! */


/* free a float vector allocated with vector() */
void free_vector(v, nl, nh)
     float *v;
     long nl, nh;
{
  free((FREE_ARG) (v+nl-NR_END));
}


/* free a int vector allocated with ivector() */
void free_ivector(v, nl, nh)
     int *v;
     long nl, nh;
{
  free((FREE_ARG) (v+nl-NR_END));
}

/* free a char vector allocated with cvector() */
void free_cvector(v, nl, nh)
     char *v;
     long nl, nh;
{
  free((FREE_ARG) (v+nl-NR_END));
}

/* free a long vector allocated with lvector() */
void free_lvector(v, nl, nh)
     long *v, nl, nh;
{
  free((FREE_ARG) (v+nl-NR_END));
}

/* free a double vector allocated with dvector() */
void free_dvector(v, nl, nh)
     double *v;
     long nl, nh;
{
  free((FREE_ARG) (v+nl-NR_END));
}

/* free a float matrix allocated with matrix() */
void free_matrix(m, nrl, nrh, ncl, nch)
     float **m;
     long nrl, nrh, ncl, nch;
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/* free a double matrix allocated with dmatrix() */
void free_dmatrix(m, nrl, nrh, ncl, nch)
     double **m;
     long nrl, nrh, ncl, nch;
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/* free a int matrix allocated with imatrix() */
void free_imatrix(m, nrl, nrh, ncl, nch)
     int **m;
     long nrl, nrh, ncl, nch;
{
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}

/* free a submatrix allocated by submatrix() */
void free_submatrix(b, nrl, nrh, ncl, nch)
     float **b;
     long nrl, nrh, ncl, nch;
{
  free((FREE_ARG) (b+nrl-NR_END));
}


/* free a matrix allocated by converted_matrix() */
void free_converte_matrix(b, nrl, nrh, ncl, nch)
     float **b; 
     long nrl, nrh,  ncl,  nch;
{
  free((FREE_ARG) (b+nrl-NR_END));
}


/* free_f3tensor  was not copied !! */






















